#include "smgbalib.h"

// this is generated with pcx2gba.exe from a indexed colour pcx image
#include "bitmaps\wallpaper.h"
#include "bitmaps\ballsprite.h"

int main() {

	// get ready for some displayin'
	Initialize();
	
	// we will use the palette in the wallpaper image! If you load wallpaper.pcx in an 
	// editing program such as GIMP, you will notice how I left colours 0 as black and 1 as white
	// This is so that transparent places seem "empty" (they use colour 0), and text will be white
	// (text output always uses colour 1 of the current palette)
	
	// this palette is conveniently created by pcx2gba.exe, along with the actual image data
	SetPalette( (u16*)wallpaperPalette );
	
	// these are the collision boundaries

	// ( / 4 ) is because we're only displaying a fourth of the total width of the strip
	// ( / 2 ) is because of how we're addressing the video memory internally (16bit operations on 8bit organization)
	u16 w = ( 240 - ballsprite_WIDTH/4) / 2; 
	u16 h = 160-ballsprite_HEIGHT;
	
	// current ball position
	int ballx = 20, bally = 20;

	// current ball direction
	u16 direction = 0;

	// this flag tells us if we should play a sound
	u16 playSound = 0;

	// C major scale
	u16 notes[] = { NOTE_C4, NOTE_D4, NOTE_E4, NOTE_F4, NOTE_G4, NOTE_A4, NOTE_B4, NOTE_C5 };
	// index of current note in the above array. We start at 0, which is a C
	u16 currentNote = 0;

	// current video frame counter (increments every time the screen refreshes)
	u16 currentFrame = 0;

	// this stores the left X coordinate of the current frame INSIDE
	// ballsprite.pcx, and it goes from 0 to 3 (4 animation frames)
	u16 spriteCoordX = 0;
	
	// hold number of total bounces
	u16 bounces = 0;

	// main loop - should never exit it
	while(1) {
		currentFrame++;

		// draw wallpaper!
		DrawFullScreenOpaque( (u16*)wallpaperData );

		// Since we can use only one palette, I suggest creating bitmaps based on the same palette
		// perhaps by copying a main pcx file, and then editing it, scaling it, but maintaining
		// its palette
		

		// (currentFrame/2) is meant to slow animation a bit
		// (% 4) moves makes the frame value go from 0 to 3 and then back again
		// (* 24) gets the actual X value in our ball animation strip
		//
		// therefore, spriteCoordX moves through the values 0, 24, 48, 72 repeatedly
		spriteCoordX = ((currentFrame/2) % 4) * 24;

		DrawBitmapSectorTransparent( ballx, bally, // where on screen are we drawing this?
									 ballsprite_HEIGHT, ballsprite_WIDTH, // total height and width of strip
									 spriteCoordX, 0, // starting from 0, 24, 48, 72 left, and 0 top 
									 24, 24, // we display 24x24 rectangle frames
									 (u16*)ballspriteData);
		
		char result[] = "           "; // allocate some space for the string :)
		Itoa( result, bounces );
		WriteStringNarrow( 30, 144, "bounces: " );
		WriteStringWide( 55, 144, result );

		WriteStringNarrow( 20, 152, "smgbalib animation demo!" );

		// flip buffers, bringing the back one (on which we've been drawring) to the screen
		Flip();

		// Done drawing... now calculate new position and direction
		// for the ball!
		//
		// direction values:
		//
		//       1       0
		//        \     /
		//        /     \
		//       2       3
		//

		// move ball appropriately
		if(direction == 0 ) {ballx+=2;bally-=2;}
		if(direction == 1 ) {ballx-=2;bally-=2;}
		if(direction == 2 ) {ballx-=2;bally+=2;}
		if(direction == 3 ) {ballx+=2;bally+=2;}
		
		// check for a bounce (which would require changing direction)
		if( direction == 0 ) {
			if( ballx >= w && bally <= 0 ) {
				direction = 2;
				playSound = 1;
			} else if ( ballx >= w ) {
				direction = 1;
				playSound = 1;
			} else if ( bally <= 0 ) {
				direction = 3;
				playSound = 1;
			}
		}

		if( direction == 1 ) {
			if( ballx <= 0 && bally <= 0 ) {
				direction = 3;
				playSound = 1;
			} else if ( ballx <= 0 ) {
				direction = 0;
				playSound = 1;
			} else if ( bally <= 0 ) {
				direction = 2;
				playSound = 1;
			}
		}

		if( direction == 2 ) {
			if( ballx <= 0 && bally >= h ) {
				direction = 0;
				playSound = 1;
			} else if ( ballx <= 0 ) {
				direction = 3;
				playSound = 1;
			} else if ( bally >= h ) {
				direction = 1;
				playSound = 1;
			}
		}

		if( direction == 3 ) {
			if( ballx >= w && bally >= h ) {
				direction = 1;
				playSound = 1;
			} else if ( ballx >= w ) {
				direction = 2;
				playSound = 1;
			} else if ( bally >= h ) {
				direction = 0;
				playSound = 1;
			}
		}
		// Done moving and bouncing ball

		// do we have to play a sound now?
		if( playSound ) {
			// next note
			currentNote = currentNote < 7 ? currentNote+1 : 0;
			PlayNote( notes[currentNote], 64 );
			playSound = 0;

			bounces++;
		}
		
	}
	
	return 0;
}


